@@ -19,13 +19,16 @@ import (
 	"testing"
 	"time"
 
-	"github.com/knative/pkg/apis"
-	"github.com/knative/pkg/apis/duck"
-	duckv1beta1 "github.com/knative/pkg/apis/duck/v1beta1"
-	apitest "github.com/knative/pkg/apis/testing"
-	"github.com/knative/serving/pkg/apis/autoscaling"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/runtime/schema"
+
+	"knative.dev/pkg/apis"
+	"knative.dev/pkg/apis/duck"
+	duckv1 "knative.dev/pkg/apis/duck/v1"
+	apitestv1 "knative.dev/pkg/apis/testing/v1"
+	"knative.dev/pkg/ptr"
+	"knative.dev/serving/pkg/apis/autoscaling"
 )
 
 func TestPodAutoscalerDuckTypes(t *testing.T) {
@@ -34,7 +37,7 @@ func TestPodAutoscalerDuckTypes(t *testing.T) {
 		t    duck.Implementable
 	}{{
 		name: "conditions",
-		t:    &duckv1beta1.Conditions{},
+		t:    &duckv1.Conditions{},
 	}}
 
 	for _, test := range tests {
@@ -61,6 +64,7 @@ func TestGeneration(t *testing.T) {
 }
 
 func TestCanScaleToZero(t *testing.T) {
+	now := time.Now()
 	cases := []struct {
 		name   string
 		status PodAutoscalerStatus
@@ -74,8 +78,8 @@ func TestCanScaleToZero(t *testing.T) {
 	}, {
 		name: "active condition",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionTrue,
 				}},
@@ -86,8 +90,8 @@ func TestCanScaleToZero(t *testing.T) {
 	}, {
 		name: "inactive condition (no LTT)",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionFalse,
 					// No LTT = beginning of time, so for sure we can.
@@ -99,12 +103,12 @@ func TestCanScaleToZero(t *testing.T) {
 	}, {
 		name: "inactive condition (LTT longer than grace period ago)",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionFalse,
 					LastTransitionTime: apis.VolatileTime{
-						Inner: metav1.NewTime(time.Now().Add(-30 * time.Second)),
+						Inner: metav1.NewTime(now.Add(-30 * time.Second)),
 					},
 					// LTT = 30 seconds ago.
 				}},
@@ -115,12 +119,12 @@ func TestCanScaleToZero(t *testing.T) {
 	}, {
 		name: "inactive condition (LTT less than grace period ago)",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionFalse,
 					LastTransitionTime: apis.VolatileTime{
-						Inner: metav1.NewTime(time.Now().Add(-10 * time.Second)),
+						Inner: metav1.NewTime(now.Add(-10 * time.Second)),
 					},
 					// LTT = 10 seconds ago.
 				}},
@@ -132,87 +136,181 @@ func TestCanScaleToZero(t *testing.T) {
 
 	for _, tc := range cases {
 		t.Run(tc.name, func(t *testing.T) {
-			if e, a := tc.result, tc.status.CanScaleToZero(tc.grace); e != a {
+			if e, a := tc.result, tc.status.CanScaleToZero(now, tc.grace); e != a {
 				t.Errorf("%q expected: %v got: %v", tc.name, e, a)
 			}
 		})
 	}
 }
 
-func TestCanMarkInactive(t *testing.T) {
+func TestActiveFor(t *testing.T) {
+	now := time.Now()
 	cases := []struct {
 		name   string
 		status PodAutoscalerStatus
-		result bool
-		idle   time.Duration
+		result time.Duration
 	}{{
 		name:   "empty status",
 		status: PodAutoscalerStatus{},
-		result: false,
-		idle:   10 * time.Second,
+		result: -1,
+	}, {
+		name: "unknown condition",
+		status: PodAutoscalerStatus{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
+					Type:   PodAutoscalerConditionActive,
+					Status: corev1.ConditionUnknown,
+				}},
+			},
+		},
+		result: -1,
 	}, {
 		name: "inactive condition",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionFalse,
 				}},
 			},
 		},
-		result: false,
-		idle:   10 * time.Second,
+		result: -1,
 	}, {
 		name: "active condition (no LTT)",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionTrue,
 					// No LTT = beginning of time, so for sure we can.
 				}},
 			},
 		},
-		result: true,
-		idle:   10 * time.Second,
+		result: time.Since(time.Time{}),
 	}, {
 		name: "active condition (LTT longer than idle period ago)",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionTrue,
 					LastTransitionTime: apis.VolatileTime{
-						Inner: metav1.NewTime(time.Now().Add(-30 * time.Second)),
+						Inner: metav1.NewTime(now.Add(-30 * time.Second)),
+					},
+					// LTT = 30 seconds ago.
+				}},
+			},
+		},
+		result: 30 * time.Second,
+	}, {
+		name: "active condition (LTT less than idle period ago)",
+		status: PodAutoscalerStatus{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
+					Type:   PodAutoscalerConditionActive,
+					Status: corev1.ConditionTrue,
+					LastTransitionTime: apis.VolatileTime{
+						Inner: metav1.NewTime(now.Add(-10 * time.Second)),
+					},
+					// LTT = 10 seconds ago.
+				}},
+			},
+		},
+		result: 10 * time.Second,
+	}}
+
+	for _, tc := range cases {
+		if got, want := tc.status.ActiveFor(now), tc.result; absDiff(got, want) > 10*time.Millisecond {
+			t.Errorf("ActiveFor = %v, want: %v", got, want)
+		}
+	}
+}
+
+func absDiff(a, b time.Duration) time.Duration {
+	a -= b
+	if a < 0 {
+		a *= -1
+	}
+	return a
+}
+
+func TestCanFailActivation(t *testing.T) {
+	now := time.Now()
+	cases := []struct {
+		name   string
+		status PodAutoscalerStatus
+		result bool
+		grace  time.Duration
+	}{{
+		name:   "empty status",
+		status: PodAutoscalerStatus{},
+		result: false,
+		grace:  10 * time.Second,
+	}, {
+		name: "active condition",
+		status: PodAutoscalerStatus{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
+					Type:   PodAutoscalerConditionActive,
+					Status: corev1.ConditionTrue,
+				}},
+			},
+		},
+		result: false,
+		grace:  10 * time.Second,
+	}, {
+		name: "activating condition (no LTT)",
+		status: PodAutoscalerStatus{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
+					Type:   PodAutoscalerConditionActive,
+					Status: corev1.ConditionUnknown,
+					// No LTT = beginning of time, so for sure we can.
+				}},
+			},
+		},
+		result: true,
+		grace:  10 * time.Second,
+	}, {
+		name: "activating condition (LTT longer than grace period ago)",
+		status: PodAutoscalerStatus{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
+					Type:   PodAutoscalerConditionActive,
+					Status: corev1.ConditionUnknown,
+					LastTransitionTime: apis.VolatileTime{
+						Inner: metav1.NewTime(now.Add(-30 * time.Second)),
 					},
 					// LTT = 30 seconds ago.
 				}},
 			},
 		},
 		result: true,
-		idle:   10 * time.Second,
+		grace:  10 * time.Second,
 	}, {
-		name: "active condition (LTT less than idle period ago)",
+		name: "activating condition (LTT less than grace period ago)",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
-					Status: corev1.ConditionTrue,
+					Status: corev1.ConditionUnknown,
 					LastTransitionTime: apis.VolatileTime{
-						Inner: metav1.NewTime(time.Now().Add(-10 * time.Second)),
+						Inner: metav1.NewTime(now.Add(-10 * time.Second)),
 					},
 					// LTT = 10 seconds ago.
 				}},
 			},
 		},
 		result: false,
-		idle:   30 * time.Second,
+		grace:  30 * time.Second,
 	}}
 
 	for _, tc := range cases {
-		if e, a := tc.result, tc.status.CanMarkInactive(tc.idle); e != a {
-			t.Errorf("%q expected: %v got: %v", tc.name, e, a)
-		}
+		t.Run(tc.name, func(t *testing.T) {
+			if e, a := tc.result, tc.status.CanFailActivation(now, tc.grace); e != a {
+				t.Errorf("%q expected: %v got: %v", tc.name, e, a)
+			}
+		})
 	}
 }
 
@@ -228,8 +326,8 @@ func TestIsActivating(t *testing.T) {
 	}, {
 		name: "active=unknown",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionUnknown,
 				}},
@@ -239,8 +337,8 @@ func TestIsActivating(t *testing.T) {
 	}, {
 		name: "active=true",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionTrue,
 				}},
@@ -250,8 +348,8 @@ func TestIsActivating(t *testing.T) {
 	}, {
 		name: "active=false",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionTrue,
 				}},
@@ -279,8 +377,8 @@ func TestIsReady(t *testing.T) {
 	}, {
 		name: "Different condition type should not be ready",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionTrue,
 				}},
@@ -290,8 +388,8 @@ func TestIsReady(t *testing.T) {
 	}, {
 		name: "False condition status should not be ready",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionReady,
 					Status: corev1.ConditionFalse,
 				}},
@@ -301,8 +399,8 @@ func TestIsReady(t *testing.T) {
 	}, {
 		name: "Unknown condition status should not be ready",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionReady,
 					Status: corev1.ConditionUnknown,
 				}},
@@ -312,8 +410,8 @@ func TestIsReady(t *testing.T) {
 	}, {
 		name: "Missing condition status should not be ready",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type: PodAutoscalerConditionReady,
 				}},
 			},
@@ -322,8 +420,8 @@ func TestIsReady(t *testing.T) {
 	}, {
 		name: "True condition status should be ready",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionReady,
 					Status: corev1.ConditionTrue,
 				}},
@@ -333,8 +431,8 @@ func TestIsReady(t *testing.T) {
 	}, {
 		name: "Multiple conditions with ready status should be ready",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionTrue,
 				}, {
@@ -347,8 +445,8 @@ func TestIsReady(t *testing.T) {
 	}, {
 		name: "Multiple conditions with ready status false should not be ready",
 		status: PodAutoscalerStatus{
-			Status: duckv1beta1.Status{
-				Conditions: duckv1beta1.Conditions{{
+			Status: duckv1.Status{
+				Conditions: duckv1.Conditions{{
 					Type:   PodAutoscalerConditionActive,
 					Status: corev1.ConditionTrue,
 				}, {
@@ -372,57 +470,43 @@ func TestTargetAnnotation(t *testing.T) {
 		name       string
 		pa         *PodAutoscaler
 		wantTarget float64
-		wantOk     bool
+		wantOK     bool
 	}{{
 		name:       "not present",
 		pa:         pa(map[string]string{}),
 		wantTarget: 0,
-		wantOk:     false,
+		wantOK:     false,
 	}, {
 		name: "present",
 		pa: pa(map[string]string{
 			autoscaling.TargetAnnotationKey: "1",
 		}),
 		wantTarget: 1,
-		wantOk:     true,
+		wantOK:     true,
 	}, {
-		name: "invalid zero",
+		name: "present float",
 		pa: pa(map[string]string{
-			autoscaling.TargetAnnotationKey: "0",
+			autoscaling.TargetAnnotationKey: "19.82",
 		}),
-		wantTarget: 0,
-		wantOk:     false,
+		wantTarget: 19.82,
+		wantOK:     true,
 	}, {
 		name: "invalid format",
 		pa: pa(map[string]string{
 			autoscaling.TargetAnnotationKey: "sandwich",
 		}),
 		wantTarget: 0,
-		wantOk:     false,
-	}, {
-		name: "invalid negative",
-		pa: pa(map[string]string{
-			autoscaling.TargetAnnotationKey: "-1",
-		}),
-		wantTarget: 0,
-		wantOk:     false,
-	}, {
-		name: "invalid overflow int32",
-		pa: pa(map[string]string{
-			autoscaling.TargetAnnotationKey: "100000000000000000000",
-		}),
-		wantTarget: 0,
-		wantOk:     false,
+		wantOK:     false,
 	}}
 
 	for _, tc := range cases {
 		t.Run(tc.name, func(t *testing.T) {
-			gotTarget, gotOk := tc.pa.Target()
+			gotTarget, gotOK := tc.pa.Target()
 			if gotTarget != tc.wantTarget {
 				t.Errorf("got target: %v wanted: %v", gotTarget, tc.wantTarget)
 			}
-			if gotOk != tc.wantOk {
-				t.Errorf("got ok: %v wanted %v", gotOk, tc.wantOk)
+			if gotOK != tc.wantOK {
+				t.Errorf("got ok: %v wanted %v", gotOK, tc.wantOK)
 			}
 		})
 	}
@@ -430,58 +514,73 @@ func TestTargetAnnotation(t *testing.T) {
 
 func TestScaleBounds(t *testing.T) {
 	cases := []struct {
-		name    string
-		pa      *PodAutoscaler
-		wantMin int32
-		wantMax int32
+		name         string
+		min          string
+		max          string
+		reachability ReachabilityType
+		wantMin      int32
+		wantMax      int32
 	}{{
-		name: "present",
-		pa: pa(map[string]string{
-			autoscaling.MinScaleAnnotationKey: "1",
-			autoscaling.MaxScaleAnnotationKey: "100",
-		}),
+		name:    "present",
+		min:     "1",
+		max:     "100",
 		wantMin: 1,
 		wantMax: 100,
 	}, {
 		name:    "absent",
-		pa:      pa(map[string]string{}),
 		wantMin: 0,
 		wantMax: 0,
 	}, {
-		name: "only min",
-		pa: pa(map[string]string{
-			autoscaling.MinScaleAnnotationKey: "1",
-		}),
+		name:    "only min",
+		min:     "1",
 		wantMin: 1,
 		wantMax: 0,
 	}, {
-		name: "only max",
-		pa: pa(map[string]string{
-			autoscaling.MaxScaleAnnotationKey: "1",
-		}),
+		name:    "only max",
+		max:     "1",
 		wantMin: 0,
 		wantMax: 1,
 	}, {
-		name: "malformed",
-		pa: pa(map[string]string{
-			autoscaling.MinScaleAnnotationKey: "ham",
-			autoscaling.MaxScaleAnnotationKey: "sandwich",
-		}),
+		name:         "reachable",
+		min:          "1",
+		max:          "100",
+		reachability: ReachabilityReachable,
+		wantMin:      1,
+		wantMax:      100,
+	}, {
+		name:         "unreachable",
+		min:          "1",
+		max:          "100",
+		reachability: ReachabilityUnreachable,
+		wantMin:      0,
+		wantMax:      100,
+	}, {
+		name:    "malformed",
+		min:     "ham",
+		max:     "sandwich",
 		wantMin: 0,
 		wantMax: 0,
 	}, {
-		name: "too small",
-		pa: pa(map[string]string{
-			autoscaling.MinScaleAnnotationKey: "-1",
-			autoscaling.MaxScaleAnnotationKey: "-1",
-		}),
+		name:    "too small",
+		min:     "-1",
+		max:     "-1",
 		wantMin: 0,
 		wantMax: 0,
 	}}
 
 	for _, tc := range cases {
 		t.Run(tc.name, func(t *testing.T) {
-			min, max := tc.pa.ScaleBounds()
+			pa := pa(map[string]string{})
+			if tc.min != "" {
+				pa.Annotations[autoscaling.MinScaleAnnotationKey] = tc.min
+			}
+			if tc.max != "" {
+				pa.Annotations[autoscaling.MaxScaleAnnotationKey] = tc.max
+			}
+			pa.Spec.Reachability = tc.reachability
+
+			min, max := pa.ScaleBounds()
+
 			if min != tc.wantMin {
 				t.Errorf("got min: %v wanted: %v", min, tc.wantMin)
 			}
@@ -507,7 +606,7 @@ func TestMarkResourceNotOwned(t *testing.T) {
 func TestMarkResourceFailedCreation(t *testing.T) {
 	pa := &PodAutoscalerStatus{}
 	pa.MarkResourceFailedCreation("doesn't", "matter")
-	apitest.CheckConditionFailed(pa.duck(), PodAutoscalerConditionActive, t)
+	apitestv1.CheckConditionFailed(pa.duck(), PodAutoscalerConditionActive, t)
 
 	active := pa.GetCondition("Active")
 	if active.Status != corev1.ConditionFalse {
@@ -528,17 +627,17 @@ func TestClass(t *testing.T) {
 		pa: pa(map[string]string{
 			autoscaling.ClassAnnotationKey: autoscaling.KPA,
 		}),
-		want: "kpa.autoscaling.knative.dev",
+		want: autoscaling.KPA,
 	}, {
 		name: "hpa class",
 		pa: pa(map[string]string{
 			autoscaling.ClassAnnotationKey: autoscaling.HPA,
 		}),
-		want: "hpa.autoscaling.knative.dev",
+		want: autoscaling.HPA,
 	}, {
 		name: "default class",
 		pa:   pa(map[string]string{}),
-		want: "kpa.autoscaling.knative.dev",
+		want: autoscaling.KPA,
 	}, {
 		name: "custom class",
 		pa: pa(map[string]string{
@@ -603,43 +702,36 @@ func TestWindowAnnotation(t *testing.T) {
 		name       string
 		pa         *PodAutoscaler
 		wantWindow time.Duration
-		wantOk     bool
+		wantOK     bool
 	}{{
 		name:       "not present",
 		pa:         pa(map[string]string{}),
 		wantWindow: 0,
-		wantOk:     false,
+		wantOK:     false,
 	}, {
 		name: "present",
 		pa: pa(map[string]string{
 			autoscaling.WindowAnnotationKey: "120s",
 		}),
 		wantWindow: time.Second * 120,
-		wantOk:     true,
+		wantOK:     true,
 	}, {
-		name: "invalid too small",
+		name: "invalid",
 		pa: pa(map[string]string{
-			autoscaling.WindowAnnotationKey: "1s",
+			autoscaling.WindowAnnotationKey: "365d",
 		}),
 		wantWindow: 0,
-		wantOk:     false,
-	}, {
-		name: "invalid format",
-		pa: pa(map[string]string{
-			autoscaling.WindowAnnotationKey: "sandwich",
-		}),
-		wantWindow: 0,
-		wantOk:     false,
+		wantOK:     false,
 	}}
 
 	for _, tc := range cases {
 		t.Run(tc.name, func(t *testing.T) {
-			gotWindow, gotOk := tc.pa.Window()
+			gotWindow, gotOK := tc.pa.Window()
 			if gotWindow != tc.wantWindow {
 				t.Errorf("%q expected target: %v got: %v", tc.name, tc.wantWindow, gotWindow)
 			}
-			if gotOk != tc.wantOk {
-				t.Errorf("%q expected ok: %v got %v", tc.name, tc.wantOk, gotOk)
+			if gotOK != tc.wantOK {
+				t.Errorf("%q expected ok: %v got %v", tc.name, tc.wantOK, gotOK)
 			}
 		})
 	}
@@ -650,50 +742,76 @@ func TestPanicWindowPercentageAnnotation(t *testing.T) {
 		name           string
 		pa             *PodAutoscaler
 		wantPercentage float64
-		wantOk         bool
+		wantOK         bool
 	}{{
 		name:           "not present",
 		pa:             pa(map[string]string{}),
 		wantPercentage: 0.0,
-		wantOk:         false,
+		wantOK:         false,
 	}, {
 		name: "present",
 		pa: pa(map[string]string{
 			autoscaling.PanicWindowPercentageAnnotationKey: "10.0",
 		}),
 		wantPercentage: 10.0,
-		wantOk:         true,
-	}, {
-		name: "too large",
-		pa: pa(map[string]string{
-			autoscaling.PanicWindowPercentageAnnotationKey: "150.0",
-		}),
-		wantPercentage: 0.0,
-		wantOk:         false,
-	}, {
-		name: "too small",
-		pa: pa(map[string]string{
-			autoscaling.PanicWindowPercentageAnnotationKey: "0.0",
-		}),
-		wantPercentage: 0.0,
-		wantOk:         false,
+		wantOK:         true,
 	}, {
 		name: "malformed",
 		pa: pa(map[string]string{
 			autoscaling.PanicWindowPercentageAnnotationKey: "sandwich",
 		}),
 		wantPercentage: 0.0,
-		wantOk:         false,
+		wantOK:         false,
 	}}
 
 	for _, tc := range cases {
 		t.Run(tc.name, func(t *testing.T) {
-			gotPercentage, gotOk := tc.pa.PanicWindowPercentage()
+			gotPercentage, gotOK := tc.pa.PanicWindowPercentage()
 			if gotPercentage != tc.wantPercentage {
 				t.Errorf("%q expected target: %v got: %v", tc.name, tc.wantPercentage, gotPercentage)
 			}
-			if gotOk != tc.wantOk {
-				t.Errorf("%q expected ok: %v got %v", tc.name, tc.wantOk, gotOk)
+			if gotOK != tc.wantOK {
+				t.Errorf("%q expected ok: %v got %v", tc.name, tc.wantOK, gotOK)
+			}
+		})
+	}
+}
+
+func TestTargetUtilization(t *testing.T) {
+	cases := []struct {
+		name   string
+		pa     *PodAutoscaler
+		want   float64
+		wantOK bool
+	}{{
+		name:   "not present",
+		pa:     pa(map[string]string{}),
+		want:   0.0,
+		wantOK: false,
+	}, {
+		name: "present",
+		pa: pa(map[string]string{
+			autoscaling.TargetUtilizationPercentageKey: "10.0",
+		}),
+		want:   .1,
+		wantOK: true,
+	}, {
+		name: "malformed",
+		pa: pa(map[string]string{
+			autoscaling.TargetUtilizationPercentageKey: "NPH",
+		}),
+		want:   0.0,
+		wantOK: false,
+	}}
+
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			got, gotOK := tc.pa.TargetUtilization()
+			if got, want := got, tc.want; got != want {
+				t.Errorf("%q target utilization: %v want: %v", tc.name, got, want)
+			}
+			if gotOK != tc.wantOK {
+				t.Errorf("%q expected ok: %v got %v", tc.name, tc.wantOK, gotOK)
 			}
 		})
 	}
@@ -704,36 +822,29 @@ func TestPanicThresholdPercentage(t *testing.T) {
 		name           string
 		pa             *PodAutoscaler
 		wantPercentage float64
-		wantOk         bool
+		wantOK         bool
 	}{{
 		name:           "not present",
 		pa:             pa(map[string]string{}),
 		wantPercentage: 0.0,
-		wantOk:         false,
+		wantOK:         false,
 	}, {
 		name: "present",
 		pa: pa(map[string]string{
 			autoscaling.PanicThresholdPercentageAnnotationKey: "300.0",
 		}),
 		wantPercentage: 300.0,
-		wantOk:         true,
-	}, {
-		name: "too small",
-		pa: pa(map[string]string{
-			autoscaling.PanicThresholdPercentageAnnotationKey: "100.0",
-		}),
-		wantPercentage: 0.0,
-		wantOk:         false,
+		wantOK:         true,
 	}}
 
 	for _, tc := range cases {
 		t.Run(tc.name, func(t *testing.T) {
-			gotPercentage, gotOk := tc.pa.PanicThresholdPercentage()
+			gotPercentage, gotOK := tc.pa.PanicThresholdPercentage()
 			if gotPercentage != tc.wantPercentage {
 				t.Errorf("%q expected target: %v got: %v", tc.name, tc.wantPercentage, gotPercentage)
 			}
-			if gotOk != tc.wantOk {
-				t.Errorf("%q expected ok: %v got %v", tc.name, tc.wantOk, gotOk)
+			if gotOK != tc.wantOK {
+				t.Errorf("%q expected ok: %v got %v", tc.name, tc.wantOK, gotOK)
 			}
 		})
 	}
@@ -757,36 +868,119 @@ func pa(annotations map[string]string) *PodAutoscaler {
 func TestTypicalFlow(t *testing.T) {
 	r := &PodAutoscalerStatus{}
 	r.InitializeConditions()
-	apitest.CheckConditionOngoing(r.duck(), PodAutoscalerConditionActive, t)
-	apitest.CheckConditionOngoing(r.duck(), PodAutoscalerConditionReady, t)
+	apitestv1.CheckConditionOngoing(r.duck(), PodAutoscalerConditionActive, t)
+	apitestv1.CheckConditionOngoing(r.duck(), PodAutoscalerConditionReady, t)
 
 	// When we see traffic, mark ourselves active.
 	r.MarkActive()
-	apitest.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionActive, t)
-	apitest.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionReady, t)
+	apitestv1.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionActive, t)
+	apitestv1.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionReady, t)
 
 	// Check idempotency.
 	r.MarkActive()
-	apitest.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionActive, t)
-	apitest.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionReady, t)
+	apitestv1.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionActive, t)
+	apitestv1.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionReady, t)
 
 	// When we stop seeing traffic, mark outselves inactive.
 	r.MarkInactive("TheReason", "the message")
-	apitest.CheckConditionFailed(r.duck(), PodAutoscalerConditionActive, t)
+	apitestv1.CheckConditionFailed(r.duck(), PodAutoscalerConditionActive, t)
 	if !r.IsInactive() {
 		t.Errorf("IsInactive was not set.")
 	}
-	apitest.CheckConditionFailed(r.duck(), PodAutoscalerConditionReady, t)
+	apitestv1.CheckConditionFailed(r.duck(), PodAutoscalerConditionReady, t)
 
 	// When traffic hits the activator and we scale up the deployment we mark
 	// ourselves as activating.
 	r.MarkActivating("Activating", "Red team, GO!")
-	apitest.CheckConditionOngoing(r.duck(), PodAutoscalerConditionActive, t)
-	apitest.CheckConditionOngoing(r.duck(), PodAutoscalerConditionReady, t)
+	apitestv1.CheckConditionOngoing(r.duck(), PodAutoscalerConditionActive, t)
+	apitestv1.CheckConditionOngoing(r.duck(), PodAutoscalerConditionReady, t)
 
 	// When the activator successfully forwards traffic to the deployment,
 	// we mark ourselves as active once more.
 	r.MarkActive()
-	apitest.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionActive, t)
-	apitest.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionReady, t)
+	apitestv1.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionActive, t)
+	apitestv1.CheckConditionSucceeded(r.duck(), PodAutoscalerConditionReady, t)
+}
+
+func TestTargetBC(t *testing.T) {
+	cases := []struct {
+		name   string
+		pa     *PodAutoscaler
+		want   float64
+		wantOK bool
+	}{{
+		name: "not present",
+		pa:   pa(map[string]string{}),
+		want: 0.0,
+	}, {
+		name: "present",
+		pa: pa(map[string]string{
+			autoscaling.TargetBurstCapacityKey: "101.0",
+		}),
+		want:   101,
+		wantOK: true,
+	}, {
+		name: "present 0",
+		pa: pa(map[string]string{
+			autoscaling.TargetBurstCapacityKey: "0",
+		}),
+		want:   0,
+		wantOK: true,
+	}, {
+		name: "present -1",
+		pa: pa(map[string]string{
+			autoscaling.TargetBurstCapacityKey: "-1",
+		}),
+		want:   -1,
+		wantOK: true,
+	}, {
+		name: "malformed",
+		pa: pa(map[string]string{
+			autoscaling.TargetBurstCapacityKey: "NPH",
+		}),
+		want: 0.0,
+	}}
+
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			got, gotOK := tc.pa.TargetBC()
+			if got, want := got, tc.want; got != want {
+				t.Errorf("%q target burst capacity: %v want: %v", tc.name, got, want)
+			}
+			if gotOK != tc.wantOK {
+				t.Errorf("%q expected ok: %v got %v", tc.name, tc.wantOK, gotOK)
+			}
+		})
+	}
+}
+
+func TestScaleStatus(t *testing.T) {
+	pas := &PodAutoscalerStatus{}
+	if got, want := pas.GetDesiredScale(), int32(-1); got != want {
+		t.Errorf("GetDesiredScale = %d, want: %v", got, want)
+	}
+	pas.DesiredScale = ptr.Int32(19980709)
+	if got, want := pas.GetDesiredScale(), int32(19980709); got != want {
+		t.Errorf("GetDesiredScale = %d, want: %v", got, want)
+	}
+
+	if got, want := pas.GetActualScale(), int32(-1); got != want {
+		t.Errorf("GetActualScale = %d, want: %v", got, want)
+	}
+	pas.ActualScale = ptr.Int32(20060907)
+	if got, want := pas.GetActualScale(), int32(20060907); got != want {
+		t.Errorf("GetActualScale = %d, want: %v", got, want)
+	}
+}
+
+func TestPodAutoscalerGetGroupVersionKind(t *testing.T) {
+	p := &PodAutoscaler{}
+	want := schema.GroupVersionKind{
+		Group:   autoscaling.InternalGroupName,
+		Version: "v1alpha1",
+		Kind:    "PodAutoscaler",
+	}
+	if got := p.GetGroupVersionKind(); got != want {
+		t.Errorf("got: %v, want: %v", got, want)
+	}
 }
