@@ -30,13 +30,11 @@ import (
 	"knative.dev/pkg/configmap"
 	"knative.dev/pkg/controller"
 	"knative.dev/pkg/logging"
-	logtesting "knative.dev/pkg/logging/testing"
 	tracingconfig "knative.dev/pkg/tracing/config"
 	asv1a1 "knative.dev/serving/pkg/apis/autoscaling/v1alpha1"
 	"knative.dev/serving/pkg/apis/networking"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
 	"knative.dev/serving/pkg/apis/serving/v1alpha1"
-	"knative.dev/serving/pkg/autoscaler"
 	"knative.dev/serving/pkg/metrics"
 	"knative.dev/serving/pkg/network"
 	"knative.dev/serving/pkg/reconciler"
@@ -137,7 +135,7 @@ func TestReconcile(t *testing.T) {
 				MarkDeploying("Deploying")),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "InternalError", "inducing failure for create podautoscalers"),
+			Eventf(corev1.EventTypeWarning, "InternalError", `failed to create PA "create-pa-failure": inducing failure for create podautoscalers`),
 		},
 		Key: "foo/create-pa-failure",
 	}, {
@@ -163,7 +161,8 @@ func TestReconcile(t *testing.T) {
 				MarkDeploying("Deploying")),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "InternalError", "inducing failure for create deployments"),
+			Eventf(corev1.EventTypeWarning, "InternalError",
+				`failed to create deployment "create-user-deploy-failure-deployment": inducing failure for create deployments`),
 		},
 		Key: "foo/create-user-deploy-failure",
 	}, {
@@ -237,7 +236,8 @@ func TestReconcile(t *testing.T) {
 			Object: deploy(t, "foo", "failure-update-deploy"),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "InternalError", "inducing failure for update deployments"),
+			Eventf(corev1.EventTypeWarning, "InternalError",
+				`failed to update deployment "failure-update-deploy-deployment": inducing failure for update deployments`),
 		},
 		Key: "foo/failure-update-deploy",
 	}, {
@@ -383,7 +383,7 @@ func TestReconcile(t *testing.T) {
 			Object: pa("foo", "fix-mutated-pa-fail", WithReachability(asv1a1.ReachabilityUnknown)),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "InternalError", "inducing failure for update podautoscalers"),
+			Eventf(corev1.EventTypeWarning, "InternalError", `failed to update PA "fix-mutated-pa-fail": inducing failure for update podautoscalers`),
 		},
 		Key: "foo/fix-mutated-pa-fail",
 	}, {
@@ -465,7 +465,7 @@ func TestReconcile(t *testing.T) {
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: rev("foo", "pod-error",
-				WithLogURL, AllUnknownConditions, MarkContainerExiting(5, "I failed man!")),
+				WithLogURL, AllUnknownConditions, MarkContainerExiting(5,  v1alpha1.RevisionContainerExitingMessage("I failed man!"))),
 		}},
 		Key: "foo/pod-error",
 	}, {
@@ -551,7 +551,6 @@ func TestReconcile(t *testing.T) {
 		Key: "foo/missing-owners",
 	}}
 
-	defer logtesting.ClearAll()
 	table.Test(t, MakeFactory(func(ctx context.Context, listers *Listers, cmw configmap.Watcher) controller.Reconciler {
 		return &Reconciler{
 			Base:                reconciler.NewBase(ctx, controllerAgentName, cmw),
@@ -670,7 +669,7 @@ func deploy(t *testing.T, namespace, name string, opts ...interface{}) *appsv1.D
 	// before calling MakeDeployment within Reconcile.
 	rev.SetDefaults(context.Background())
 	deployment, err := resources.MakeDeployment(rev, cfg.Logging, cfg.Tracing, cfg.Network,
-		cfg.Observability, cfg.Autoscaler, cfg.Deployment,
+		cfg.Observability, cfg.Deployment,
 	)
 
 	if err != nil {
@@ -733,8 +732,7 @@ func ReconcilerTestConfig() *config.Config {
 		Observability: &metrics.ObservabilityConfig{
 			LoggingURLTemplate: "http://logger.io/${REVISION_UID}",
 		},
-		Logging:    &logging.Config{},
-		Tracing:    &tracingconfig.Config{},
-		Autoscaler: &autoscaler.Config{},
+		Logging: &logging.Config{},
+		Tracing: &tracingconfig.Config{},
 	}
 }
